#ifndef _TRAP_H_
#define _TRAP_H_

#include <time.h>

enum field_type_t {
	FIELD_STRING,
	FIELD_NUMERIC,
	FIELD_BOOLEAN,
	FIELD_NULL,
	FIELD_TIME
};

enum result_state_t {
	STATE_UNCHANGED = -1,
	STATE_OK,
	STATE_WARNING,
	STATE_CRITICAL,
	STATE_UNKNOWN
};

enum result_notify_t {
	NOTIFY_NEW_STATE = 0,
	NOTIFY_ALWAYS = 9
};

typedef struct proc_trap proc_trap;
typedef struct proc_field proc_field;
typedef struct proc_handler proc_handler;
typedef struct proc_result proc_result;

struct proc_trap {
	unsigned int id;
	time_t timestamp;
	char *host;
	char *oid;
	proc_field *fields;
	proc_trap *next;
};

struct proc_field {
	char *oid;
	enum field_type_t type;
	union {
		char *string;
		long long number;
		short boolean;
		time_t time;
	} value;
	proc_field *next;
};

struct proc_handler {
	char *name;
	char *script;
	char *context;
	proc_handler *next;
};

struct proc_result {
	char *host;
	char *service;
	char *tag;
	enum result_state_t state;
	char *message;
	enum result_notify_t notify;
	proc_trap *trap;
	proc_handler *handler;
};

/**
 * free a proc_result. Does not free any associated
 * proc_trap - said reference is considered weak.
 */
void proc_result_free(proc_result *result);

/* free a linked list of handlers */
void proc_handler_free(proc_handler *handler);

/**
 * free a proc_trap.
 * Does free any associated proc_fields - said reference is strong.
 * Does not free the whole linked list - weak.
 */
void proc_trap_free(proc_trap *result);
/**
 * free proc_fields
 * Does free the whole linked list - strong ref.
 */
void proc_field_free(proc_field *result);

char *display_fields(const proc_field *field);
char *display_trap(const proc_trap *trap);
#endif
